@@ -4,9 +4,10 @@ set -e
 
 BELENIOS_SRC="${BELENIOS_SRC:-$PWD}"
 
-# Check that OCamlDuce is not installed
-if which ocamlduce >/dev/null; then
-    echo "Please uninstall OCamlDuce first, or remove it from your PATH."
+# Check that Dune is not installed
+# cf. https://github.com/ocaml/opam/issues/3987
+if command -v dune >/dev/null; then
+    echo "Please uninstall Dune first, or remove it from your PATH."
     exit 1
 fi
 
@@ -24,6 +25,7 @@ fi
 
 export BELENIOS_SYSROOT="${BELENIOS_SYSROOT:-$HOME/.belenios}"
 export OPAMROOT="$BELENIOS_SYSROOT/opam"
+export XDG_CACHE_HOME="$BELENIOS_SYSROOT/cache"
 
 if [ -e "$BELENIOS_SYSROOT" ]; then
     echo "$BELENIOS_SYSROOT already exists."
@@ -34,11 +36,11 @@ fi
 mkdir -p "$BELENIOS_SYSROOT/bootstrap/src"
 
 cd "$BELENIOS_SYSROOT/bootstrap/src"
-wget https://github.com/ocaml/opam/releases/download/2.0.5/opam-full-2.0.5.tar.gz
+wget https://github.com/ocaml/opam/releases/download/2.0.7/opam-full-2.0.7.tar.gz
 
 if which sha256sum >/dev/null; then
 sha256sum --check <<EOF
-776c7e64d6e24c2ef1efd1e6a71d36e007645efae94eaf860c05c1929effc76f  opam-full-2.0.5.tar.gz
+9c0dac1094ed624158fff13000cdfa8edbc96798d32b9fab40b0b5330f9490a2  opam-full-2.0.7.tar.gz
 EOF
 else
     echo "WARNING: sha256sum was not found, checking tarballs is impossible!"
@@ -50,31 +52,38 @@ echo
 echo "=-=-= Compilation and installation of OPAM =-=-="
 echo
 cd "$BELENIOS_SYSROOT/bootstrap/src"
-tar -xzf opam-full-2.0.5.tar.gz
-cd opam-full-2.0.5
+tar -xzf opam-full-2.0.7.tar.gz
+cd opam-full-2.0.7
+cp $BELENIOS_SRC/ext/opam/bootstrap-ocaml.sh $BELENIOS_SYSROOT/bootstrap/src/opam-full-2.0.7/shell
 make cold CONFIGURE_ARGS="--prefix $BELENIOS_SYSROOT/bootstrap"
 make cold-install LIBINSTALL_DIR="$BELENIOS_SYSROOT/bootstrap/lib/ocaml"
 
 echo
 echo "=-=-= Generation of env.sh =-=-="
 echo
-cat > $BELENIOS_SRC/env.sh <<EOF
+cat > $BELENIOS_SYSROOT/env.sh <<EOF
 PATH="$BELENIOS_SYSROOT/bootstrap/bin:\$PATH"; export PATH;
 OPAMROOT=$OPAMROOT; export OPAMROOT;
+XDG_CACHE_HOME=$XDG_CACHE_HOME; export XDG_CACHE_HOME;
 eval \$(opam env)
 EOF
+ln -sf $BELENIOS_SYSROOT/env.sh $BELENIOS_SRC/env.sh
 
 echo
 echo "=-=-= Initialization of OPAM root =-=-="
 echo
-opam init --bare --no-setup
-opam switch create 4.06.1
+cd "$BELENIOS_SYSROOT"
+git clone https://github.com/ocaml/opam-repository.git
+cd opam-repository
+git reset --hard ea0d9f1d30e13bb1db009b79a39bbe7a95b0217e
+opam init --bare --no-setup -k git "$BELENIOS_SYSROOT/opam-repository"
+opam switch create 4.11.1 ocaml-base-compiler.4.11.1 --jobs=1
 eval $(opam env)
 
 echo
 echo "=-=-= Installation of Belenios build-dependencies =-=-="
 echo
-opam install --yes dune=1.6.3 atdgen zarith cryptokit uuidm calendar cmdliner sqlite3 ssl=0.5.7 js_of_ocaml=3.3.0 eliom=6.3.0 csv
+opam install --yes dune atdgen zarith cryptokit calendar cmdliner sqlite3 csv eliom gettext-camomile
 
 echo
 echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
