@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,6 +19,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Belenios_platform
 open Platform
 open Serializable_builtin_t
 open Serializable_core_j
@@ -53,8 +54,6 @@ let has_nh_questions e =
       | Question.Homomorphic _ -> false
     ) e.e_params.e_questions
 
-let check_modulo p x = Z.(geq x zero && lt x p)
-
 (** Homomorphic elections *)
 
 module Make (W : ELECTION_DATA) (M : RANDOM) = struct
@@ -77,6 +76,7 @@ module Make (W : ELECTION_DATA) (M : RANDOM) = struct
 
   type plaintext = int array array
   type ballot = elt Serializable_t.ballot
+  type weighted_ballot = int * ballot
 
   (** Fiat-Shamir non-interactive zero-knowledge proofs of
       knowledge *)
@@ -168,7 +168,12 @@ module Make (W : ELECTION_DATA) (M : RANDOM) = struct
   let process_ballots bs =
     SArray (
         Array.mapi (fun i q ->
-            Q.process_ciphertexts q (Array.map (fun b -> Q.extract_ciphertexts q b.answers.(i)) bs)
+            Q.process_ciphertexts q
+              (Array.map
+                 (fun (w, b) ->
+                   w, Q.extract_ciphertexts q b.answers.(i)
+                 ) bs
+              )
           ) election.e_params.e_questions
       )
 
@@ -264,33 +269,125 @@ module Make (W : ELECTION_DATA) (M : RANDOM) = struct
 
   type result = elt Serializable_t.election_result
 
-  type combinator = factor list -> elt shape
+  module Combinator = Trustees.MakeCombinator (G)
 
-  let compute_result ?shuffles num_tallied encrypted_tally partial_decryptions combinator =
-    let factors = combinator partial_decryptions in
-    let results = Shape.map2 (fun {beta; _} f ->
-      beta / f
-    ) encrypted_tally factors in
-    let result =
-      match results with
-      | SAtomic _ ->
-         invalid_arg "Election.compute_result: cannot compute result"
-      | SArray xs ->
-         SArray (Array.map2 (Q.compute_result ~num_tallied) election.e_params.e_questions xs)
-    in
-    {num_tallied; encrypted_tally; shuffles; partial_decryptions; result}
+  let compute_result ?shuffles ?shufflers num_tallied encrypted_tally partial_decryptions trustees =
+    let check = check_factor encrypted_tally in
+    match Combinator.combine_factors trustees check partial_decryptions with
+    | Ok factors ->
+       let results =
+         Shape.map2 (fun {beta; _} f ->
+             beta / f
+           ) encrypted_tally factors
+       in
+       let result =
+         match results with
+         | SAtomic _ ->
+            invalid_arg "Election.compute_result: cannot compute result"
+         | SArray xs ->
+            SArray (Array.map2 (Q.compute_result ~num_tallied) election.e_params.e_questions xs)
+       in
+       Ok {num_tallied; encrypted_tally; shuffles; shufflers; partial_decryptions; result}
+    | Error e -> Error e
 
-  let check_result combinator r =
+  let check_result trustees r =
     let {encrypted_tally; partial_decryptions; result; _} = r in
     check_ciphertext encrypted_tally &&
-    let factors = combinator partial_decryptions in
-    let results = Shape.map2 (fun {beta; _} f ->
-      beta / f
-    ) encrypted_tally factors in
-    match results, result with
-    | SArray xs, SArray rs ->
-       Array.forall3 Q.check_result election.e_params.e_questions xs rs
-    | _, _ -> false
+    let check = check_factor encrypted_tally in
+    match Combinator.combine_factors trustees check partial_decryptions with
+    | Error _ -> false
+    | Ok factors ->
+       let results =
+         Shape.map2 (fun {beta; _} f ->
+             beta / f
+           ) encrypted_tally factors
+       in
+       match results, result with
+       | SArray xs, SArray rs ->
+          Array.forall3 Q.check_result election.e_params.e_questions xs rs
+       | _, _ -> false
 
   let extract_tally r = r.result
 end
+
+(** Computing checksums *)
+
+let compute_checksums ~election result_or_shuffles ~trustees ~public_credentials =
+  let ec_election = sha256_b64 election in
+  let ec_public_credentials = sha256_b64 public_credentials in
+  let tc_of_tpk k =
+    let tc_checksum = sha256_b64 (Yojson.Safe.to_string k.trustee_public_key) in
+    let tc_name = k.trustee_name in
+    {tc_checksum; tc_name}
+  in
+  let trustees = trustees_of_string Yojson.Safe.read_json trustees in
+  let ec_trustees =
+    trustees
+    |> List.map
+         (function
+          | `Single k -> [tc_of_tpk k]
+          | `Pedersen _ -> []
+         )
+    |> List.flatten
+  in
+  let ec_trustees_threshold =
+    trustees
+    |> List.map
+         (function
+          | `Single _ -> []
+          | `Pedersen p ->
+             let ts_trustees =
+               List.combine (Array.to_list p.t_verification_keys) (Array.to_list p.t_certs)
+               |> List.map
+                    (fun (key, cert) ->
+                      {
+                        ttc_name = key.trustee_name;
+                        ttc_pki_key = sha256_b64 cert.s_message;
+                        ttc_verification_key =
+                          sha256_b64 (Yojson.Safe.to_string key.trustee_public_key);
+                      }
+                    )
+             in
+             [{ ts_trustees; ts_threshold = p.t_threshold }]
+         )
+    |> List.flatten
+    |> (fun x -> Some x)
+  in
+  let combine shuffles shufflers =
+    match shuffles, shufflers with
+    | Some shuffles, Some shufflers ->
+       List.combine shuffles shufflers
+       |> List.map
+            (fun (shuffle, shuffler) ->
+              let shuffle = string_of_shuffle Yojson.Safe.write_json shuffle in
+              let tc_checksum = sha256_b64 shuffle in
+              {tc_checksum; tc_name = shuffler}
+            )
+       |> (fun x -> Some x)
+    | Some shuffles, None ->
+       shuffles
+       |> List.map
+            (fun shuffle ->
+              let shuffle = string_of_shuffle Yojson.Safe.write_json shuffle in
+              let tc_checksum = sha256_b64 shuffle in
+              {tc_checksum; tc_name = None}
+            )
+       |> (fun x -> Some x)
+    | None, None -> None
+    | _, _ -> failwith "ill-formed result"
+  in
+  let ec_shuffles, ec_encrypted_tally =
+    match result_or_shuffles with
+    | `Nothing -> None, None
+    | `Shuffles (shuffles, shufflers) ->
+       let shuffles = List.map (shuffle_of_string Yojson.Safe.read_json) shuffles in
+       combine (Some shuffles) shufflers, None
+    | `Result result ->
+       let result = election_result_of_string Yojson.Safe.read_json result in
+       let tally = string_of_encrypted_tally Yojson.Safe.write_json result.encrypted_tally in
+       combine result.shuffles result.shufflers, Some (sha256_b64 tally)
+  in
+  {
+    ec_election; ec_pki = None; ec_trustees; ec_trustees_threshold;
+    ec_public_credentials; ec_shuffles; ec_encrypted_tally
+  }
