@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -23,11 +23,19 @@
 
 open Signatures
 
-val of_string : string -> Yojson.Safe.json election
-val get_group : Yojson.Safe.json election -> (module ELECTION_DATA)
+val of_string : string -> Yojson.Safe.t election
+val get_group : Yojson.Safe.t election -> (module ELECTION_DATA)
 
 val has_nh_questions : 'a election -> bool
 
 module Make (W : ELECTION_DATA) (M : RANDOM) :
   ELECTION with type elt = W.G.t and type 'a m = 'a M.t
 (** Implementation of {!Signatures.ELECTION}. *)
+
+val compute_checksums :
+  election:string ->
+  [`Nothing
+  | `Shuffles of string list * string option list option
+  | `Result of string] ->
+  trustees:string -> public_credentials:string ->
+  Serializable_t.election_checksums
