@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,14 +19,13 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Belenios_platform
 open Platform
 open Serializable_j
 open Common
 
 (** Helper functions *)
 
-let check_modulo p x = Z.(geq x zero && lt x p)
-
 let map_and_concat_with_commas f xs =
   let n = Array.length xs in
   let res = Buffer.create (n * 1024) in
@@ -55,7 +54,7 @@ module type GROUP = Signatures.GROUP
   with type t = Z.t
   and type group = ff_params
 
-let unsafe_make group =
+let make group =
   let {p; q; g; embedding} = group in
   let module G = struct
     open Z
@@ -121,7 +120,7 @@ let unsafe_make group =
 
     let hash prefix xs =
       let x = prefix ^ (map_and_concat_with_commas Z.to_string xs) in
-      let z = Z.of_string_base 16 (sha256_hex x) in
+      let z = Z.of_hex (sha256_hex x) in
       Z.(z mod q)
 
     let compare = Z.compare
@@ -130,7 +129,7 @@ let unsafe_make group =
       let cofactor = Z.((p - one) / q) in
       fun i ->
       let s = Printf.sprintf "ggen|%d" i in
-      let h = Z.of_string_base 16 (sha256_hex s) in
+      let h = Z.of_hex (sha256_hex s) in
       let h = Z.powm h cofactor p in
       (* it is very unlikely (but theoretically possible) that one of the following assertions fails *)
       assert (Z.(compare h zero) <> 0);
@@ -143,7 +142,3 @@ let unsafe_make group =
     let write_group = write_ff_params
 
   end in (module G : GROUP)
-
-let make group =
-  if check_params group then unsafe_make group
-  else invalid_arg "incorrect finite field parameters"
