@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2019 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,6 +19,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Belenios_platform
 open Platform
 open Common
 open Signatures_core
@@ -28,8 +29,6 @@ open Question_h_t
 
 (** Helper functions *)
 
-let check_modulo p x = Z.(geq x zero && lt x p)
-
 let question_length q =
   Array.length q.q_answers + match q.q_blank with
                              | Some true -> 1
@@ -429,6 +428,14 @@ module Make (M : RANDOM) (G : GROUP) = struct
 
   let process_ciphertexts q es =
     let neutral = SArray (Array.make (question_length q) (SAtomic dummy_ciphertext)) in
+    let ( * ) = Shape.map2 eg_combine in
+    let rec power b n =
+      if n > 0 then (
+        let x = power b Stdlib.(n / 2) in
+        (if n mod 2 = 1 then b else neutral) * x * x
+      ) else neutral
+    in
+    let es = Array.map (fun (w, b) -> power b w) es in
     Array.fold_left (Shape.map2 eg_combine) neutral es
 
   let compute_result ~num_tallied =
