@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -21,6 +21,7 @@
 
 (** Signatures *)
 
+open Belenios_platform
 open Platform
 open Serializable_t
 
@@ -55,6 +56,11 @@ module type ELECTION_DATA = sig
   val election : G.t election
 end
 
+type combination_error =
+  | MissingPartialDecryption
+  | NotEnoughPartialDecryptions
+  | UnusedPartialDecryption
+
 (** Cryptographic primitives for an election with homomorphic tally. *)
 module type ELECTION = sig
 
@@ -88,6 +94,8 @@ module type ELECTION = sig
       answers and cryptographic proofs that they satisfy the election
       constraints. *)
 
+  type weighted_ballot = int * ballot
+
   val create_ballot : ?sk:private_key -> plaintext -> ballot m
   (** [create_ballot r answers] creates a ballot, or raises
       [Invalid_argument] if [answers] doesn't satisfy the election
@@ -100,7 +108,7 @@ module type ELECTION = sig
 
   (** {2 Tally} *)
 
-  val process_ballots : ballot array -> elt encrypted_tally
+  val process_ballots : weighted_ballot array -> elt encrypted_tally
 
   val extract_nh_ciphertexts : elt encrypted_tally -> elt nh_ciphertexts
   val merge_nh_ciphertexts : elt nh_ciphertexts -> elt encrypted_tally -> elt encrypted_tally
@@ -128,14 +136,15 @@ module type ELECTION = sig
   (** The election result. It contains the needed data to validate the
       result from the encrypted tally. *)
 
-  type combinator = factor list -> elt shape
-
-  val compute_result : ?shuffles:elt shuffle list -> int -> elt Serializable_t.ciphertext shape -> factor list -> combinator -> result
+  val compute_result :
+    ?shuffles:elt shuffle list -> ?shufflers:shuffler list ->
+    int -> elt Serializable_t.ciphertext shape -> factor list -> elt trustees ->
+    (result, combination_error) Stdlib.result
   (** Combine the encrypted tally and the factors from all trustees to
       produce the election result. The first argument is the number of
       tallied ballots. May raise [Invalid_argument]. *)
 
-  val check_result : combinator -> result -> bool
+  val check_result : elt trustees -> result -> bool
 
   val extract_tally : result -> int shape
   (** Extract the plaintext result of the election. *)
@@ -177,12 +186,6 @@ module type PEDERSEN = sig
   val step5 : certs -> string -> vinput -> elt voutput m
   val step5_check : certs -> int -> polynomial array -> elt voutput -> bool
   val step6 : certs -> polynomial array -> elt voutput array -> elt threshold_parameters
-
-  val check : elt threshold_parameters -> bool
-  val combine : elt threshold_parameters -> elt
-
-  type checker = elt -> elt partial_decryption -> bool
-  val combine_factors : checker -> elt threshold_parameters -> elt partial_decryption list -> elt shape
 end
 
 module type MIXNET = sig
