@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,6 +19,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Belenios_platform
 open Platform
 open Serializable_t
 open Signatures
@@ -37,15 +38,6 @@ module MakeSimple (G : GROUP) (M : RANDOM) : sig
   (** [prove x] returns the public key associated to [x] and a zero-
       knowledge proof of its knowledge. *)
 
-  val check : G.t trustee_public_key -> bool
-  (** Check a public key and its proof. *)
-
-  val combine : G.t trustee_public_key array -> G.t
-  (** Combine all public key shares into an election public key. *)
-
-  type checker = G.t -> G.t partial_decryption -> bool
-  val combine_factors : checker -> G.t array -> G.t partial_decryption list -> G.t shape
-
 end
 (** Simple distributed generation of an election public key. *)
 
@@ -73,3 +65,21 @@ module MakePedersen (G : GROUP) (M : RANDOM)
                         and type public_key = G.t) :
   PEDERSEN with type 'a m = 'a M.t
             and type elt = G.t
+
+val string_of_combination_error : combination_error -> string
+
+module MakeCombinator (G : GROUP) : sig
+
+  val check : G.t trustees -> bool
+  (** Check consistency of a set of trustees. *)
+
+  val combine_keys : G.t trustees -> G.t
+  (** Compute the public key associated to a set of trustees. *)
+
+  val combine_factors :
+    G.t trustees ->
+    (G.t -> G.t partial_decryption -> bool) ->
+    G.t partial_decryption list -> (G.t shape, combination_error) result
+  (** Compute synthetic decryption factors. *)
+
+end
