@@ -1,6 +1,8 @@
 function init_prng () {
+    var random = belenios.sjcl.random;
+
     // Start SJCL built-in collectors
-    sjcl.random.startCollectors();
+    random.startCollectors();
 
     // Seed from window.crypto if present
     var cryptoObj = window.crypto || window.msCrypto; // for IE 11
@@ -9,7 +11,7 @@ function init_prng () {
         var bytes = new Uint32Array(n);
         cryptoObj.getRandomValues(bytes);
         for (var i = 0; i < n; i++) {
-            sjcl.random.addEntropy(bytes[i], 32);
+            random.addEntropy(bytes[i], 32);
         }
         if (console) {
             console.log("PRNG successfully initialized using crypto object");
