@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -205,8 +205,9 @@ let random_string = Cryptokit.Random.string
 
 module Z = struct
   include Z
+  let of_hex x = of_string_base 16 x
   let ( =% ) = equal
-  let bit_length x = Pervasives.(String.length (to_bits x) * 8)
+  let bit_length x = Stdlib.(String.length (to_bits x) * 8)
 
   let powm = powm_sec (* Warning: no efforts have been made to be
                          constant time in the rest of the code. *)
