@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,7 +19,10 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Belenios
 open Signatures
+open Belenios_platform
+open Belenios_tool_common
 open Serializable_j
 open Platform
 open Common
@@ -45,6 +48,16 @@ let lines_of_stdin () =
       | exception End_of_file -> None
     )
 
+let chars_of_stdin () =
+  let buf = Buffer.create 1024 in
+  let rec loop () =
+    match input_char stdin with
+    | c -> Buffer.add_char buf c; loop ()
+    | exception End_of_file -> ()
+  in
+  loop ();
+  Buffer.contents buf
+
 let string_of_file f =
   lines_of_file f |> stream_to_list |> String.concat "\n"
 
@@ -80,7 +93,7 @@ let common_man = [
   `P "This command is part of the Belenios command-line tool.";
   `P "To get more help on a specific subcommand, run:";
   `P "$(b,belenios-tool) $(i,COMMAND) $(b,--help)";
-  `P "See $(i,http://www.belenios.org/).";
+  `P "See $(i,https://www.belenios.org/).";
 ]
 
 let get_mandatory_opt name = function
@@ -98,6 +111,27 @@ module type CMDLINER_MODULE = sig
   val cmds : (unit Cmdliner.Term.t * Cmdliner.Term.info) list
 end
 
+module Shasum : CMDLINER_MODULE = struct
+
+  let main () =
+    wrap_main (fun () -> chars_of_stdin () |> sha256_b64 |> print_endline)
+
+  let sha256_b64_cmd =
+    let doc = "compute SHA256 of standard input and encode it in Base64Compact" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command compute the SHA256 of standard input and encode it in Base64Compact. This computation is frequent when auditing an election. This single shell command is equivalent to the following shell pipeline:";
+        `Pre "sha256sum | xxd -r -p | base64 | tr -d \"=\"";
+        `P "but does not need each individual command to be available.";
+      ] @ common_man
+    in
+    Term.(ret (pure main $ pure ())),
+    Term.info "sha256-b64" ~doc ~man
+
+  let cmds = [sha256_b64_cmd]
+
+end
+
 let group_t =
   let doc = "Take group parameters from file $(docv)." in
   Arg.(value & opt (some file) None & info ["group"] ~docv:"GROUP" ~doc)
@@ -295,20 +329,16 @@ module Election : CMDLINER_MODULE = struct
 
   module MakeGetters (X : sig val dir : string end) = struct
 
-    let get_threshold () =
-      let file = "threshold.json" in
-      Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (string_of_file (X.dir / file)) with _ -> None
-
-    let get_public_keys () =
-      load_from_file (fun x -> x) (X.dir/"public_keys.jsons") |>
-      Option.map Array.of_list
-
     let get_public_creds () =
       let file = "public_creds.txt" in
       Printf.eprintf "I: loading %s...\n%!" file;
       try Some (lines_of_file (X.dir / file)) with _ -> None
 
+    let get_trustees () =
+      let file = "trustees.json" in
+      Printf.eprintf "I: loading %s...\n%!" file;
+      try Some (string_of_file (X.dir / file)) with _ -> None
+
     let get_ballots () =
       let file = "ballots.jsons" in
       Printf.eprintf "I: loading %s...\n%!" file;
@@ -350,10 +380,10 @@ module Election : CMDLINER_MODULE = struct
         | Some u ->
            if not (
              download dir u "election.json" &&
-             download dir u "public_keys.jsons" &&
+             download dir u "trustees.json" &&
              download dir u "public_creds.txt" &&
-             download dir u "ballots.jsons" &&
-             download dir u "result.json"
+             (download dir u "ballots.jsons" || true) &&
+             (download dir u "result.json" || download dir u "shuffles.jsons" || true)
            ) then
              Printf.eprintf "W: some errors occurred while downloading\n%!";
       in
@@ -381,6 +411,7 @@ module Election : CMDLINER_MODULE = struct
         in
         print_endline (X.vote (Some privcred) ballot)
       | `Decrypt privkey ->
+        X.verify ();
         let privkey =
           match load_from_file (fun x -> x) privkey with
           | Some [privkey] -> privkey
@@ -407,6 +438,10 @@ module Election : CMDLINER_MODULE = struct
       | `Shuffle ->
          let s = X.shuffle_ciphertexts () in
          print_endline s
+      | `Checksums ->
+         X.checksums () |> print_endline
+      | `ComputeVoters privcreds ->
+         X.compute_voters privcreds |> List.iter print_endline
       end;
       if cleanup then rm_rf dir
     )
@@ -431,6 +466,11 @@ module Election : CMDLINER_MODULE = struct
     let the_info = Arg.info ["decryption-key"] ~docv:"KEY" ~doc in
     Arg.(value & opt (some file) None the_info)
 
+  let privcreds_t =
+    let doc = "Read private credentials from file $(docv)." in
+    let the_info = Arg.info ["privcreds"] ~docv:"PRIVCREDS" ~doc in
+    Arg.(value & opt (some file) None the_info)
+
   let vote_cmd =
     let doc = "create a ballot" in
     let man = [
@@ -499,7 +539,48 @@ module Election : CMDLINER_MODULE = struct
     Term.(ret (pure main $ url_t $ optdir_t $ pure `Shuffle)),
     Term.info "shuffle" ~doc ~man
 
-  let cmds = [vote_cmd; verify_cmd; decrypt_cmd; tdecrypt_cmd; validate_cmd; shuffle_cmd]
+  let checksums_cmd =
+    let doc = "compute checksums" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command computes checksums needed to audit an election.";
+      ] @ common_man
+    in
+    Term.(ret (pure main $ url_t $ optdir_t $ pure `Checksums)),
+    Term.info "checksums" ~doc ~man
+
+  let compute_voters_cmd =
+    let doc = "compute actual voters" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command computes the list of voters that actually voted in an election, from the list of ballots and private credentials.";
+      ] @ common_man
+    in
+    let main =
+      Term.pure
+        (fun u d privcreds ->
+          let privcreds =
+            get_mandatory_opt "--privcreds" privcreds
+            |> lines_of_file
+            |> stream_to_list
+          in
+          main u d (`ComputeVoters privcreds)
+        )
+    in
+    Term.(ret (main $ url_t $ optdir_t $ privcreds_t)),
+    Term.info "compute-voters" ~doc ~man
+
+  let cmds =
+    [
+      vote_cmd;
+      verify_cmd;
+      decrypt_cmd;
+      tdecrypt_cmd;
+      validate_cmd;
+      shuffle_cmd;
+      checksums_cmd;
+      compute_voters_cmd;
+    ]
 
 end
 
@@ -508,7 +589,6 @@ module Credgen : CMDLINER_MODULE = struct
 
   let params_priv = "private credentials with ids", ".privcreds", 0o400
   let params_pub = "public credentials", ".pubcreds", 0o444
-  let params_hash = "hashed public credentials with ids", ".hashcreds", 0o400
 
   let save (info, ext, perm) basename things =
     let fname = basename ^ ext in
@@ -544,18 +624,15 @@ module Credgen : CMDLINER_MODULE = struct
       | `Derive c ->
         print_endline (R.derive c)
       | `Generate ids ->
-        let privs, pubs, hashs =
-          List.fold_left (fun (privs, pubs, hashs) id ->
-            let priv, pub, hash = R.generate () in
-            let priv = id ^ " " ^ priv and hash = id ^ " " ^ hash in
-            priv::privs, pub::pubs, hash::hashs
-          ) ([], [], []) ids
+        let privs, pubs = R.generate ids in
+        let privs =
+          List.combine ids privs
+          |> List.map (fun (id, priv) -> id ^ " " ^ priv)
         in
         let timestamp = Printf.sprintf "%.0f" (Unix.time ()) in
         let base = dir / timestamp in
-        save params_priv base (List.rev privs);
-        save params_pub base (List.sort compare pubs);
-        save params_hash base (List.rev hashs)
+        save params_priv base privs;
+        save params_pub base pubs
     )
 
   let count_t =
@@ -564,7 +641,7 @@ module Credgen : CMDLINER_MODULE = struct
     Arg.(value & opt (some int) None the_info)
 
   let file_t =
-    let doc = "Read identities from $(docv) and generate an additional $(i,T.hashcreds) with identities associated with hashed public credentials. These hashed public credentials are used by the hotline to update a public credential on the web server. One credential will be generated for each line of $(docv)." in
+    let doc = "Read identities from $(docv). One credential will be generated for each line of $(docv)." in
     let the_info = Arg.info ["file"] ~docv:"FILE" ~doc in
     Arg.(value & opt (some file) None the_info)
 
@@ -586,6 +663,58 @@ module Credgen : CMDLINER_MODULE = struct
 
 end
 
+module Mktrustees : CMDLINER_MODULE = struct
+  let main dir =
+    wrap_main
+      (fun () ->
+        let get_public_keys () =
+          Some (lines_of_file (dir / "public_keys.jsons") |> stream_to_list)
+        in
+        let get_threshold () =
+          let fn = dir / "threshold.json" in
+          if Sys.file_exists fn then Some (string_of_file fn) else None
+        in
+        let get_trustees () =
+          let singles =
+            match get_public_keys () with
+            | None -> []
+            | Some t ->
+               t
+               |> List.map (trustee_public_key_of_string Yojson.Safe.read_json)
+               |> List.map (fun x -> `Single x)
+          in
+          let pedersens =
+            match get_threshold () with
+            | None -> []
+            | Some t ->
+               t
+               |> threshold_parameters_of_string Yojson.Safe.read_json
+               |> (fun x -> [`Pedersen x])
+          in
+          match singles @ pedersens with
+          | [] -> failwith "trustees are missing"
+          | trustees -> string_of_trustees Yojson.Safe.write_json trustees
+        in
+        let trustees = get_trustees () in
+        let oc = open_out (dir / "trustees.json") in
+        output_string oc trustees;
+        output_char oc '\n';
+        close_out oc
+      )
+
+  let mktrustees_cmd =
+    let doc = "create a trustee parameter file" in
+    let man = [
+      `S "DESCRIPTION";
+      `P "This command reads $(i,public_keys.jsons) and $(i,threshold.json) (if any). It then generates an $(i,trustees.json) file.";
+    ] @ common_man in
+    Term.(ret (pure main $ dir_t)),
+    Term.info "mktrustees" ~doc ~man
+
+  let cmds = [mktrustees_cmd]
+
+end
+
 module Mkelection : CMDLINER_MODULE = struct
   open Tool_mkelection
 
@@ -595,11 +724,12 @@ module Mkelection : CMDLINER_MODULE = struct
         let group = get_mandatory_opt "--group" group |> string_of_file
         let uuid = get_mandatory_opt "--uuid" uuid
         let template = get_mandatory_opt "--template" template |> string_of_file
-        let get_public_keys () =
-          Some (lines_of_file (dir / "public_keys.jsons") |> stream_to_list |> Array.of_list)
-        let get_threshold () =
-          let fn = dir / "threshold.json" in
-          if Sys.file_exists fn then Some (string_of_file fn) else None
+        let get_trustees () =
+          let fn = dir / "trustees.json" in
+          if Sys.file_exists fn then
+            string_of_file fn
+          else
+            failwith "trustees are missing"
       end in
       let module R = (val make (module P : PARAMS) : S) in
       let params = R.mkelection () in
@@ -657,7 +787,148 @@ module Verifydiff : CMDLINER_MODULE = struct
 
 end
 
-let cmds = Tkeygen.cmds @ Ttkeygen.cmds @ Election.cmds @ Credgen.cmds @ Mkelection.cmds @ Verifydiff.cmds
+module Methods : CMDLINER_MODULE = struct
+
+  let schulze nchoices =
+    wrap_main (fun () ->
+        let ballots = chars_of_stdin () |> condorcet_ballots_of_string in
+        let nchoices =
+          if nchoices = 0 then
+            if Array.length ballots > 0 then Array.length ballots.(0) else 0
+          else nchoices
+        in
+        if nchoices <= 0 then
+          failcmd "invalid --nchoices parameter (or could not infer it)"
+        else
+          ballots
+          |> Schulze.compute ~nchoices
+          |> string_of_schulze_result
+          |> print_endline
+      )
+
+  let mj nchoices ngrades =
+    wrap_main (fun () ->
+        let ballots = chars_of_stdin () |> mj_ballots_of_string in
+        let nchoices =
+          if nchoices = 0 then
+            if Array.length ballots > 0 then Array.length ballots.(0) else 0
+          else nchoices
+        in
+        if nchoices <= 0 then
+          failcmd "invalid --nchoices parameter (or could not infer it)"
+        else
+          let ngrades =
+            match ngrades with
+            | None -> failcmd "--ngrades is missing"
+            | Some i -> if i > 0 then i else failcmd "invalid --ngrades paramater"
+          in
+          ballots
+          |> Majority_judgment.compute ~nchoices ~ngrades
+          |> string_of_mj_result
+          |> print_endline
+      )
+
+  let stv nseats =
+    wrap_main (fun () ->
+        let nseats =
+          match nseats with
+          | None -> failcmd "--nseats is missing"
+          | Some i -> if i > 0 then i else failcmd "invalid --nseats parameter"
+        in
+        chars_of_stdin ()
+        |> stv_raw_ballots_of_string
+        |> Stv.compute ~nseats
+        |> string_of_stv_result
+        |> print_endline
+      )
+
+  let nchoices_t =
+    let doc = "Number of choices. If 0, try to infer it." in
+    Arg.(value & opt int 0 & info ["nchoices"] ~docv:"N" ~doc)
+
+  let ngrades_t =
+    let doc = "Number of grades." in
+    Arg.(value & opt (some int) None & info ["ngrades"] ~docv:"G" ~doc)
+
+  let nseats_t =
+    let doc = "Number of seats." in
+    Arg.(value & opt (some int) None & info ["nseats"] ~docv:"N" ~doc)
+
+  let schulze_cmd =
+    let doc = "compute Schulze result" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command reads on standard input JSON-formatted ballots and interprets them as Condorcet rankings on $(i,N) choices. It then computes the result according to the Schulze method and prints it on standard output.";
+      ] @ common_man
+    in
+    Term.(ret (pure schulze $ nchoices_t)),
+    Term.info "method-schulze" ~doc ~man
+
+  let mj_cmd =
+    let doc = "compute Majority Judgment result" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command reads on standard input JSON-formatted ballots and interprets them as grades (ranging from 1 (best) to $(i,G) (worst)) given to $(i,N) choices. It then computes the result according to the Majority Judgment method and prints it on standard output.";
+      ] @ common_man
+    in
+    Term.(ret (pure mj $ nchoices_t $ ngrades_t)),
+    Term.info "method-majority-judgment" ~doc ~man
+
+  let stv_cmd =
+    let doc = "compute Single Transferable Vote result" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command reads on standard input JSON-formatted ballots and interprets them as rankings of choices (ranging from 1 (best) to $(i,X) (worst)). It then computes the result according to the Single Transferable Vote method and prints it on standard output.";
+      ] @ common_man
+    in
+    Term.(ret (pure stv $ nseats_t)),
+    Term.info "method-stv" ~doc ~man
+
+  let cmds = [schulze_cmd; mj_cmd; stv_cmd]
+
+end
+
+module GenerateToken : CMDLINER_MODULE = struct
+
+  let main length =
+    wrap_main (fun () ->
+        let module X = MakeGenerateToken (DirectRandom) in
+        X.generate_token ~length ()
+        |> print_endline
+      )
+
+  let length_t =
+    let doc = "Token length." in
+    Arg.(value & opt int 14 & info ["length"] ~docv:"L" ~doc)
+
+  let generate_token_cmd =
+    let doc = "generate a token" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command generates a random token suitable for an election identifier.";
+      ] @ common_man
+    in
+    Term.(ret (pure main $ length_t)),
+    Term.info "generate-token" ~doc ~man
+
+  let cmds = [generate_token_cmd]
+
+end
+
+let cmds =
+  List.flatten
+    [
+      Shasum.cmds;
+      Tkeygen.cmds;
+      Ttkeygen.cmds;
+      Election.cmds;
+      Credgen.cmds;
+      Mktrustees.cmds;
+      Mkelection.cmds;
+      Verifydiff.cmds;
+      Methods.cmds;
+      GenerateToken.cmds;
+    ]
 
 let default_cmd =
   let open Belenios_version in
