@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -20,12 +20,17 @@
 (**************************************************************************)
 
 open Js_of_ocaml
+open Js_of_ocaml_lwt
+open Belenios_platform
+open Belenios
+open Belenios_tool_js_common
 open Platform
 open Serializable_builtin_t
 open Serializable_j
 open Signatures
 open Common
 open Tool_js_common
+open Tool_js_i18n.Gettext
 
 let encryptBallot params cred plaintext () =
   let module P = (val params : ELECTION_DATA) in
@@ -54,7 +59,14 @@ let progress_step n =
     new_##.style##.fontWeight := Js.string "bold"
   in ()
 
-let appendHomomorphicQuestion div num_questions i q answers =
+type questionWidget =
+  {
+    div : Dom_html.divElement Js.t;
+    extract_answers : unit -> (int array * Dom_html.divElement Js.t) option;
+  }
+
+let createHomomorphicQuestionWidget q =
+  let div = Dom_html.createDiv document in
   let open Question_h_t in
   let () =
     let c = Dom_html.createH2 document in
@@ -64,20 +76,16 @@ let appendHomomorphicQuestion div num_questions i q answers =
   in
   let () =
     let c = Dom_html.createDiv document in
-    let fmt = Scanf.format_from_string
-      (get_content "question_header") "%d%d%d%d"
-    in
-    let s = Printf.sprintf fmt
-      (i + 1) num_questions q.q_min q.q_max
-    in
+    let s = Printf.sprintf (f_ "Select between %d and %d answer(s)") q.q_min q.q_max in
     let t = document##createTextNode (Js.string s) in
     Dom.appendChild c t;
     Dom.appendChild div c
   in
   let q_answers = match q.q_blank with
-    | Some true -> Array.append [|get_content "str_blank_vote"|] q.q_answers
+    | Some true -> Array.append [|s_ "Blank vote"|] q.q_answers
     | _ -> q.q_answers
   in
+  let answers = Array.make (Array.length q_answers) 0 in
   let () =
     let choices = Dom_html.createDiv document in
     choices##.className := Js.string "answer_div";
@@ -114,12 +122,10 @@ let appendHomomorphicQuestion div num_questions i q answers =
   let check_constraints () =
     let check_min_max total =
       if total < q.q_min then (
-        let fmt = Scanf.format_from_string (get_content "at_least") "%d" in
-        Printf.ksprintf alert fmt q.q_min;
+        Printf.ksprintf alert (f_ "You must select at least %d answer(s)") q.q_min;
         false
       ) else if total > q.q_max then (
-        let fmt = Scanf.format_from_string (get_content "at_most") "%d" in
-        Printf.ksprintf alert fmt q.q_max;
+        Printf.ksprintf alert (f_ "You must select at most %d answer(s)") q.q_max;
         false
       ) else true
     in
@@ -129,16 +135,52 @@ let appendHomomorphicQuestion div num_questions i q answers =
        let total = Array.fold_left (+) 0 answers' in
        if answers.(0) > 0 then (
          if total <> 0 then
-           (alert (get_content "no_other_blank"); false)
+           (alert (s_ "No other choices are allowed when voting blank"); false)
          else true
        ) else check_min_max total
     | _ ->
        let total = Array.fold_left (+) 0 answers in
        check_min_max total
   in
-  check_constraints
+  let create_summary () =
+    let e = Dom_html.createDiv document in
+    let h = Dom_html.createH3 document in
+    let t = document##createTextNode (Js.string q.q_question) in
+    Dom.appendChild h t;
+    Dom.appendChild e h;
+    let ul = Dom_html.createUl document in
+    let checked = ref 0 in
+    Array.iteri
+      (fun i a ->
+        if a > 0 then (
+          incr checked;
+          let li = Dom_html.createLi document in
+          let text = match q.q_blank with
+            | Some true -> if i = 0 then s_ "Blank vote" else q.q_answers.(i-1)
+            | _ -> q.q_answers.(i)
+          in
+          let t = document##createTextNode (Js.string text) in
+          Dom.appendChild li t;
+          Dom.appendChild ul li;
+        )
+      ) answers;
+    if !checked = 0 then (
+      let t = document##createTextNode (Js.string (s_ "(nothing)")) in
+      Dom.appendChild ul t
+    );
+    Dom.appendChild e ul;
+    e
+  in
+  let extract_answers () =
+    if check_constraints () then
+      Some (Array.copy answers, create_summary ())
+    else
+      None
+  in
+  {div; extract_answers}
 
-let appendNonHomomorphicQuestion div q answers =
+let createNonHomomorphicQuestionWidget q =
+  let div = Dom_html.createDiv document in
   let open Question_nh_t in
   let () =
     let c = Dom_html.createH2 document in
@@ -146,6 +188,7 @@ let appendNonHomomorphicQuestion div q answers =
     Dom.appendChild c t;
     Dom.appendChild div c
   in
+  let answers = Array.make (Array.length q.q_answers) 0 in
   let inputs =
     let choices = Dom_html.createDiv document in
     choices##.className := Js.string "answer_div";
@@ -158,19 +201,15 @@ let appendNonHomomorphicQuestion div q answers =
       let t = document##createTextNode (Js.string (" " ^ a)) in
       input##.onchange :=
         Dom_html.handler (fun _ ->
-            let x =
-              try
-                let x = int_of_string (Js.to_string input##.value) in
-                if x < 0 then raise Exit;
-                if x > 255 then raise Exit;
-                x
-              with _ ->
-                alert (get_content "alert_0_255");
-                answers.(i)
-            in
-            answers.(i) <- x;
-            input##.value := Js.string (string_of_int x);
-            Js._true
+            try
+              let x = int_of_string (Js.to_string input##.value) in
+              if x < 0 then raise Exit;
+              if x > 255 then raise Exit;
+              answers.(i) <- x;
+              Js._true
+            with _ ->
+              alert (s_ "Value must be an integer between 0 and 255.");
+              Js._false
           );
       Dom.appendChild div t;
       input, div
@@ -182,6 +221,25 @@ let appendNonHomomorphicQuestion div q answers =
     Dom.appendChild div choices;
     Array.map fst choices_divs
   in
+  let () =
+    let d = Dom_html.createDiv document in
+    d##.style##.margin := Js.string "1em";
+    Dom.appendChild div d;
+    let div_note = Dom_html.createDiv document in
+    Dom.appendChild d div_note;
+    Dom.appendChild div_note (document##createTextNode (Js.string (s_ "Notes:")));
+    let list = Dom_html.createUl document in
+    Dom.appendChild div_note list;
+    let item1 = Dom_html.createLi document in
+    Dom.appendChild list item1;
+    Dom.appendChild item1 (document##createTextNode (Js.string (s_ "If you are asked to grade candidates (majority judgement) then 1 is the best grade, higher numbers are worse.")));
+    let item2 = Dom_html.createLi document in
+    Dom.appendChild list item2;
+    Dom.appendChild item2 (document##createTextNode (Js.string (s_ "If you are asked to rank candidates (Condorcet, STV, ...) then use 1 for your first choice, 2 for the second, etc.")));
+    let div_warning = Dom_html.createDiv document in
+    Dom.appendChild d div_warning;
+    Dom.appendChild div_warning (document##createTextNode (Js.string (s_ "Warning: the system will accept any integer between 0 and 255 but, according to the election rules, invalid ballots (score too high or candidates not properly ranked) will be rejected at the end of the election.")))
+  in
   let check_constraints () =
     let n = Array.length inputs - 1 in
     let rec loop i =
@@ -196,172 +254,126 @@ let appendNonHomomorphicQuestion div q answers =
           | None -> false
         in
         if not valid then (
-          alert (get_content "at_least_one_invalid");
+          alert (s_ "At least one of the answers is invalid!");
           false
         ) else loop (i + 1)
       else true
     in
     loop 0
   in
-  check_constraints
-
-let appendHomomorphicSummary e a q =
-  let open Question_h_t in
-  let h = Dom_html.createH3 document in
-  let t = document##createTextNode (Js.string q.q_question) in
-  Dom.appendChild h t;
-  Dom.appendChild e h;
-  let ul = Dom_html.createUl document in
-  let checked = ref 0 in
-  Array.iteri (fun i a ->
-      if a > 0 then (
-        incr checked;
+  let create_summary () =
+    let e = Dom_html.createDiv document in
+    let h = Dom_html.createH3 document in
+    let t = document##createTextNode (Js.string q.q_question) in
+    Dom.appendChild h t;
+    Dom.appendChild e h;
+    let ul = Dom_html.createUl document in
+    Array.iteri
+      (fun i a ->
         let li = Dom_html.createLi document in
-        let text = match q.q_blank with
-          | Some true -> if i = 0 then get_content "str_blank_vote" else q.q_answers.(i-1)
-          | _ -> q.q_answers.(i)
-        in
+        let text = Printf.sprintf "%d (%s)" a q.q_answers.(i) in
         let t = document##createTextNode (Js.string text) in
         Dom.appendChild li t;
         Dom.appendChild ul li;
-      )
-    ) a;
-  if !checked = 0 then (
-    let t = document##createTextNode (Js.string @@ get_content "str_nothing") in
-    Dom.appendChild ul t
-  );
-  Dom.appendChild e ul
-
-let appendNonHomomorphicSummary e a q =
-  let open Question_nh_t in
-  let h = Dom_html.createH3 document in
-  let t = document##createTextNode (Js.string q.q_question) in
-  Dom.appendChild h t;
-  Dom.appendChild e h;
-  let ul = Dom_html.createUl document in
-  Array.iteri (fun i a ->
-      let li = Dom_html.createLi document in
-      let text = Printf.sprintf "%d (%s)" a q.q_answers.(i) in
-      let t = document##createTextNode (Js.string text) in
-      Dom.appendChild li t;
-      Dom.appendChild ul li;
-    ) a;
-  Dom.appendChild e ul
-
-let rec createQuestionNode sk params question_div num_questions i prev (q, answers) next =
-  (* Create div element for the current question. [i] and [(q,
-     answers)] point to the current question. [List.rev prev @ [q,
-     answers] @ next] is the list of all questions. *)
-  let div = Dom_html.createDiv document in
-  let nh, check_constraints =
-    match q with
-    | Question.Homomorphic q -> false, appendHomomorphicQuestion div num_questions i q answers
-    | Question.NonHomomorphic q -> true, appendNonHomomorphicQuestion div q answers
+      ) answers;
+    Dom.appendChild e ul;
+    e
   in
-  let () =
-    (* previous button *)
-    let btns = Dom_html.createDiv document in
-    btns##.style##.textAlign := Js.string "center";
-    let () =
-      match prev with
-      | [] ->
-        (* first question, no "Previous" button *)
-        ()
-      | r :: prev ->
-        let b = Dom_html.createButton document in
-        let t = document##createTextNode (Js.string @@ get_content "str_previous") in
-        b##.onclick := Dom_html.handler (fun _ ->
-          if check_constraints () then (
-            let ndiv = createQuestionNode sk params
-              question_div num_questions (i - 1) prev r ((q, answers) :: next)
-            in
-            Dom.replaceChild question_div ndiv div;
-            Js._false
-          ) else Js._false
-        );
-        Dom.appendChild b t;
-        Dom.appendChild btns b;
-    in
-    let () =
-      (* next button *)
-      match next with
-      | [] ->
-        (* last question, the button leads to encryption page *)
-        let b = Dom_html.createButton document in
-        let t = document##createTextNode (Js.string @@ get_content "str_next") in
-        b##.onclick := Dom_html.handler (fun _ ->
-         if check_constraints () then (
-          let all = (q, answers) :: prev in
-          let all_answers = List.rev_map snd all |> Array.of_list in
-          let all_questions = List.rev_map fst all |> Array.of_list in
-          set_textarea "choices" (string_of_plaintext all_answers);
-          question_div##.style##.display := Js.string "none";
-          let () =
-            document##getElementById (Js.string "pretty_choices") >>== fun e ->
-            Array.iteri (fun i a ->
-                match all_questions.(i) with
-                | Question.Homomorphic q -> appendHomomorphicSummary e a q
-                | Question.NonHomomorphic q -> appendNonHomomorphicSummary e a q
-              ) all_answers
-          in
-          Lwt_js_events.async (encryptBallot params sk all_answers);
-          set_element_display "plaintext_div" "block";
-          progress_step 3;
-          Js._false
-         ) else Js._false
-        );
-        Dom.appendChild b t;
-        Dom.appendChild btns b
-      | r :: next ->
-        let b = Dom_html.createButton document in
-        let t = document##createTextNode (Js.string @@ get_content "str_next") in
-        b##.onclick := Dom_html.handler (fun _ ->
-          if check_constraints () then (
-            let ndiv = createQuestionNode sk params
-              question_div num_questions (i + 1) ((q, answers) :: prev) r next
-            in
-            Dom.replaceChild question_div ndiv div;
-            Js._false
-          ) else Js._false
-        );
-        Dom.appendChild b t;
-        Dom.appendChild btns b;
-    in
-    Dom.appendChild div btns
+  let extract_answers () =
+    if check_constraints () then
+      Some (Array.copy answers, create_summary ())
+    else
+      None
   in
-  let () =
-    if nh then (
-      let d = Dom_html.createDiv document in
-      d##.style##.marginTop := Js.string "1em";
-      Dom.appendChild div d;
-      Dom.appendChild d (document##createTextNode (Js.string (get_content "warning_0_255")))
-    )
-  in
-  div
+  {div; extract_answers}
 
-let addQuestions sk params qs =
-  document##getElementById (Js.string "question_div") >>== fun e ->
+let createQuestionWidget = function
+  | Question.Homomorphic q -> createHomomorphicQuestionWidget q
+  | Question.NonHomomorphic q -> createNonHomomorphicQuestionWidget q
+
+let createLabeledButton label =
+  let b = Dom_html.createButton document in
+  let t = document##createTextNode (Js.string label) in
+  Dom.appendChild b t;
+  b
+
+let arrayMapOptionGet xs =
+  try
+    Array.map
+      (function
+       | Some x -> x
+       | None -> raise Exit
+      ) xs
+    |> (fun x -> Some x)
+  with
+  | Exit -> None
+
+let appendQuestionNavigation question_div sk params qs =
+  let qs = Array.map createQuestionWidget qs in
   let n = Array.length qs in
-  let qs =
-    Array.to_list qs |>
-      List.map (fun q ->
-          match q with
-          | Question.Homomorphic x -> q, Array.make (Question_h.question_length x) 0
-          | Question.NonHomomorphic x -> q, Array.make (Array.length x.Question_nh_t.q_answers) 0
-        )
-  in
-  match qs with
-  | [] -> failwith "no questions"
-  | q :: next ->
-     let div = createQuestionNode sk params e n 0 [] q next in
-     Dom.appendChild e div
+  let answers = Array.make n None in
+  if n < 1 then failwith "no questions";
+  let cur = ref 0 in
+  Dom.appendChild question_div qs.(!cur).div;
+  let btns = Dom_html.createDiv document in
+  btns##.style##.textAlign := Js.string "center";
+  let btn_prev = createLabeledButton (s_ "Previous") in
+  let btn_next = createLabeledButton (s_ "Next") in
+  btn_prev##.id := Js.string "btn_prev";
+  btn_next##.id := Js.string "btn_next";
+  Dom.appendChild btns btn_prev;
+  Dom.appendChild btns btn_next;
+  btn_prev##.style##.display := Js.string "none";
+  btn_prev##.onclick :=
+    Dom_html.handler
+      (fun _ ->
+        answers.(!cur) <- qs.(!cur).extract_answers ();
+        match answers.(!cur) with
+        | None -> Js._false
+        | Some _ ->
+           decr cur;
+           let d = if !cur = 0 then "none" else "inline" in
+           btn_prev##.style##.display := Js.string d;
+           Dom.replaceChild question_div qs.(!cur).div qs.(!cur + 1).div;
+           Js._true
+      );
+  btn_next##.onclick :=
+    Dom_html.handler
+      (fun _ ->
+        answers.(!cur) <- qs.(!cur).extract_answers ();
+        match answers.(!cur) with
+        | None -> Js._false
+        | Some _ ->
+           if !cur < n - 1 then (
+             incr cur;
+             btn_prev##.style##.display := Js.string "inline";
+             Dom.replaceChild question_div qs.(!cur).div qs.(!cur - 1).div;
+             Js._true
+           ) else (
+             match arrayMapOptionGet answers with
+             | None -> Js._false
+             | Some answers ->
+                question_div##.style##.display := Js.string "none";
+                let plaintext = Array.map fst answers in
+                set_textarea "choices" (string_of_plaintext plaintext);
+                let () =
+                  document##getElementById (Js.string "pretty_choices") >>== fun e ->
+                  Array.iter (fun (_, a) -> Dom.appendChild e a) answers
+                in
+                Lwt_js_events.async (encryptBallot params sk plaintext);
+                set_element_display "plaintext_div" "block";
+                progress_step 3;
+                Js._true
+           )
+      );
+  Dom.appendChild question_div btns
 
 let createStartButton params intro_div qs =
   let b = Dom_html.createButton document in
   b##.style##.fontSize := Js.string "20px";
-  let t = document##createTextNode (Js.string (get_content "str_here")) in
+  let t = document##createTextNode (Js.string (s_ "here")) in
   b##.onclick := Dom_html.handler (fun _ ->
-    (match prompt (get_content "enter_cred") with
+    (match prompt (s_ "Please enter your credential:") with
     | Some cred when Credential.check cred ->
       intro_div##.style##.display := Js.string "none";
       set_element_display "question_div" "block";
@@ -369,9 +381,10 @@ let createStartButton params intro_div qs =
         Js._false
       );
       progress_step 2;
-      addQuestions cred params qs
+      document##getElementById (Js.string "question_div") >>== fun e ->
+      appendQuestionNavigation e cred params qs
     | Some _ ->
-       alert (get_content "invalid_cred")
+       alert (s_ "Invalid credential!")
     | None -> ()
     );
     Js._false
@@ -408,61 +421,69 @@ let get_prefix str =
   let n = String.length str in
   if n >= 4 then String.sub str 0 (n-4) else str
 
-let get_uuid x =
+let get_params x =
   let n = String.length x in
   if n <= 1 || String.sub x 0 1 <> "#" then
-    None
+    []
   else
-    let args = Url.decode_arguments (String.sub x 1 (n-1)) in
-    List.assoc_opt "uuid" args
+    Url.decode_arguments (String.sub x 1 (n-1))
 
 let load_uuid uuid =
-  let open Lwt_xmlHttpRequest in
-  Lwt.async (fun () ->
-      let%lwt raw =
-        let%lwt x = Printf.ksprintf get "elections/%s/election.json" uuid in
-        if x.code = 404 then (
-          let%lwt x = Printf.ksprintf get "draft/preview/%s/election.json" uuid in
-          Lwt.return x.content
-        ) else Lwt.return x.content
-      in
-      let () = set_textarea "election_params" raw in
-      Lwt.return (run_handler loadElection ())
-    )
+  let open Js_of_ocaml_lwt.XmlHttpRequest in
+  let%lwt raw =
+    let%lwt x = Printf.ksprintf get "elections/%s/election.json" uuid in
+    if x.code = 404 then (
+      let%lwt x = Printf.ksprintf get "draft/preview/%s/election.json" uuid in
+      Lwt.return x.content
+    ) else Lwt.return x.content
+  in
+  set_textarea "election_params" raw;
+  run_handler loadElection ();
+  Lwt.return_unit
 
-let load_uuid_handler _ =
-  (match get_textarea_opt "uuid" with
-   | Some uuid ->
-      let encoded = Url.encode_arguments ["uuid", uuid] in
-      Dom_html.window##.location##.hash := Js.string encoded;
-      load_uuid uuid
-   | None -> ()
-  ); Js._false
+let load_uuid_handler lang =
+  match get_textarea_opt "uuid" with
+  | Some uuid ->
+     let encoded = Url.encode_arguments ["uuid", uuid; "lang", lang] in
+     Dom_html.window##.location##.hash := Js.string encoded;
+     load_uuid uuid
+  | None -> Lwt.return_unit
 
-let load_params_handler _ =
+let load_params_handler () =
   set_element_display "div_ballot" "block";
   set_element_display "div_submit" "none";
   set_element_display "div_submit_manually" "block";
+  run_handler loadElection ();
+  Lwt.return_unit
+
+let () =
   Lwt.async (fun () ->
-      Lwt.return (run_handler loadElection ())
-    );
-  Js._false
-
-let onload_handler _ =
-  let () =
-    document##getElementById (Js.string "load_uuid") >>== fun e ->
-    e##.onclick := Dom_html.handler load_uuid_handler
-  in
-  let () =
-    document##getElementById (Js.string "load_params") >>== fun e ->
-    e##.onclick := Dom_html.handler load_params_handler;
-  in
-  let () =
-    match get_uuid (Js.to_string Dom_html.window##.location##.hash) with
-    | None ->
-       set_element_display "wait_div" "none";
-       set_element_display "election_loader" "block";
-    | Some uuid -> load_uuid uuid
-  in Js._false
-
-let () = Dom_html.window##.onload := Dom_html.handler onload_handler
+      let%lwt _ = Lwt_js_events.onload () in
+      let params = get_params (Js.to_string Dom_html.window##.location##.hash) in
+      let lang =
+        match List.assoc_opt "lang" params with
+        | Some x -> x
+        | None -> "en"
+      in
+      let%lwt () = Tool_js_i18n.init "static" "voter" lang in
+      let () =
+        document##getElementById (Js.string "load_uuid") >>== fun e ->
+        Lwt_js_events.async (fun () ->
+            let%lwt _ = Lwt_js_events.click e in
+            load_uuid_handler lang
+          )
+      in
+      let () =
+        document##getElementById (Js.string "load_params") >>== fun e ->
+        Lwt_js_events.async (fun () ->
+            let%lwt _ = Lwt_js_events.click e in
+            load_params_handler ()
+          )
+      in
+      match List.assoc_opt "uuid" params with
+      | None ->
+         set_element_display "wait_div" "none";
+         set_element_display "election_loader" "block";
+         Lwt.return_unit
+      | Some uuid -> load_uuid uuid
+    )
