@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -20,12 +20,16 @@
 (**************************************************************************)
 
 open Js_of_ocaml
+open Belenios_platform
+open Belenios_tool_common
+open Belenios_tool_js_common
 open Tool_js_common
 open Tool_credgen
+open Tool_js_i18n.Gettext
 
 let generate _ =
+  let raw = get_textarea "voters" in
   let ids =
-    let raw = get_textarea "voters" in
     let rec loop i accu =
       if i >= 0 then
         let j =
@@ -36,42 +40,42 @@ let generate _ =
         loop (j-1) (String.sub raw (j+1) (i-j) :: accu)
       else
         accu
-    in loop (String.length raw - 1) []
+    in loop (String.length raw - 2) []
   in
   let module P : PARAMS = struct
     let uuid = get_textarea "uuid"
     let group = get_textarea "group"
   end in
   let module X = (val make (module P : PARAMS) : S) in
-  let privs, pubs, hashs =
-    List.fold_left
-      (fun (privs, pubs, hashs) id ->
-       let priv, pub, hash = X.generate () in
-       let priv = id ^ " " ^ priv and hash = id ^ " " ^ hash in
-       priv::privs, pub::pubs, hash::hashs
-      ) ([], [], []) ids
+  let privs, pubs = X.generate ids in
+  let privs =
+    List.combine ids privs
+    |> List.map (fun (id, priv) -> id ^ " " ^ priv)
   in
-  let text_pks = pubs |> List.sort compare |> String.concat "\n" in
+  let text_pks = (pubs |> String.concat "\n") ^ "\n" in
   set_textarea "pks" text_pks;
-  set_download "public_creds" "text/plain" "public_creds.txt" text_pks;
-  let text_creds = (privs |> List.rev |> String.concat "\n") ^ "\n" in
+  let hash = Platform.sha256_b64 text_pks in
+  set_content "public_creds_fp" hash;
+  let text_creds = (privs |> String.concat "\n") ^ "\n" in
   set_download "creds" "text/plain" "creds.txt" text_creds;
-  let text_hashed = (hashs |> List.rev |> String.concat "\n") ^ "\n" in
-  set_download "hashed" "text/plain" "hashed.txt" text_hashed;
+  set_download "voters_txt" "text/plain" "voters.txt" raw;
   set_element_display "submit_form" "inline";
   Js._false
 
-let fill_interactivity _ =
-  let () =
-    document##getElementById (Js.string "interactivity") >>== fun e ->
-    let x = Dom_html.createDiv document in
-    Dom.appendChild e x;
-    let b = Dom_html.createButton document in
-    let t = document##createTextNode (Js.string "Generate") in
-    b##.onclick := Dom_html.handler generate;
-    Dom.appendChild b t;
-    Dom.appendChild x b
-  in Js._false
+let fill_interactivity () =
+  document##getElementById (Js.string "interactivity") >>== fun e ->
+  let x = Dom_html.createDiv document in
+  Dom.appendChild e x;
+  let b = Dom_html.createButton document in
+  let t = document##createTextNode (Js.string (s_ "Generate")) in
+  b##.onclick := Dom_html.handler generate;
+  Dom.appendChild b t;
+  Dom.appendChild x b
 
 let () =
-  Dom_html.window##.onload := Dom_html.handler fill_interactivity;
+  Lwt.async (fun () ->
+      let%lwt _ = Js_of_ocaml_lwt.Lwt_js_events.onload () in
+      let%lwt () = Tool_js_i18n.auto_init "admin" in
+      fill_interactivity ();
+      Lwt.return_unit
+    )
