@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -20,8 +20,14 @@
 (**************************************************************************)
 
 open Js_of_ocaml
+open Belenios_platform
+open Belenios
+open Belenios_tool_common
+open Belenios_tool_js_common
+open Serializable_j
 open Tool_js_common
 open Tool_tkeygen
+open Tool_js_i18n.Gettext
 
 let tkeygen _ =
   let module P : PARAMS = struct
@@ -30,21 +36,28 @@ let tkeygen _ =
   let module X = (val make (module P : PARAMS) : S) in
   let open X in
   let {id=_; priv; pub} = trustee_keygen () in
+  let hash =
+    let pub = trustee_public_key_of_string Yojson.Safe.read_json pub in
+    Platform.sha256_b64 (Yojson.Safe.to_string pub.trustee_public_key)
+  in
   set_textarea "pk" pub;
-  set_download "public_key" "application/json" "public_key.json" pub;
+  set_content "public_key_fp" hash;
   set_download "private_key" "application/json" "private_key.json" priv;
   set_element_display "submit_form" "inline";
   Js._false
 
-let fill_interactivity _ =
-  let () =
-    document##getElementById (Js.string "interactivity") >>== fun e ->
-    let b = Dom_html.createButton document in
-    let t = document##createTextNode (Js.string "Generate a new keypair") in
-    b##.onclick := Dom_html.handler tkeygen;
-    Dom.appendChild b t;
-    Dom.appendChild e b
-  in Js._false
+let fill_interactivity () =
+  document##getElementById (Js.string "interactivity") >>== fun e ->
+  let b = Dom_html.createButton document in
+  let t = document##createTextNode (Js.string (s_ "Generate a key")) in
+  b##.onclick := Dom_html.handler tkeygen;
+  Dom.appendChild b t;
+  Dom.appendChild e b
 
 let () =
-  Dom_html.window##.onload := Dom_html.handler fill_interactivity;
+  Lwt.async (fun () ->
+      let%lwt _ = Js_of_ocaml_lwt.Lwt_js_events.onload () in
+      let%lwt () = Tool_js_i18n.auto_init "admin" in
+      fill_interactivity ();
+      Lwt.return_unit
+    )
