@@ -2,8 +2,7 @@ module type PARAMS = sig
   val uuid : string
   val group : string
   val template : string
-  val get_public_keys : unit -> string array option
-  val get_threshold : unit -> string option
+  val get_trustees : unit -> string
 end
 
 module type S = sig
