@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -28,57 +28,72 @@ open Web_services
 type result = Eliom_registration.Html.result
 
 type post_login_handler =
-  uuid option -> auth_config -> (string -> unit Lwt.t) -> unit Lwt.t
+  uuid option -> auth_config -> (string -> unit Lwt.t) -> (unit, unit) Stdlib.result Lwt.t
 
 let scope = Eliom_common.default_session_scope
 
 let auth_env = Eliom_reference.eref ~scope None
 
-let run_post_login_handler auth_system f =
-  match%lwt Eliom_reference.get auth_env with
-  | None -> Printf.ksprintf failwith "%s handler was invoked without environment" auth_system
-  | Some (uuid, a, cont) ->
-     let authenticate name =
-       let%lwt () = Eliom_reference.unset auth_env in
-       let user = { user_domain = a.auth_instance; user_name = name } in
-       match uuid with
-       | None -> Eliom_reference.set Web_state.site_user (Some user)
-       | Some uuid -> Eliom_reference.set Web_state.election_user (Some (uuid, user))
-     in
-     let%lwt () = f uuid a authenticate in
-     cont ()
+let get_cont login_or_logout x =
+  let open Eliom_registration in
+  let redir = match x with
+    | `Election uuid -> Redirection (preapply ~service:election_cast_fallback uuid)
+    | `Site ContSiteHome -> Redirection home
+    | `Site ContSiteAdmin -> Redirection admin
+    | `Site (ContSiteElection uuid) ->
+       match login_or_logout with
+       | `Login -> Redirection (preapply ~service:election_admin uuid)
+       | `Logout -> Redirection (preapply ~service:election_home (uuid, ()))
+  in
+  fun () -> Redirection.send redir
 
-type pre_login_handler = auth_config -> result Lwt.t
+let restart_login service = function
+  | `Election uuid -> preapply ~service:election_login ((uuid, ()), Some service)
+  | `Site cont -> preapply ~service:site_login (Some service, cont)
+
+let run_post_login_handler ~auth_system ~state f =
+  match%lwt Eliom_reference.get auth_env with
+  | None -> Eliom_registration.Action.send ()
+  | Some (uuid, a, kind, st) ->
+     let restart_login () =
+       let service = restart_login a.auth_instance kind in
+       Pages_common.login_failed ~service ()
+       >>= Eliom_registration.Html.send ~code:401
+     in
+     if auth_system = a.auth_system && st = state then
+       let authenticate name =
+         let%lwt () = Eliom_reference.unset auth_env in
+         let user = { user_domain = a.auth_instance; user_name = name } in
+         match uuid with
+         | None -> Eliom_reference.set Web_state.site_user (Some user)
+         | Some uuid -> Eliom_reference.set Web_state.election_user (Some (uuid, user))
+       in
+       match%lwt f uuid a authenticate with
+       | Ok () -> get_cont `Login kind ()
+       | Error () -> restart_login ()
+     else
+       restart_login ()
+
+type pre_login_handler = auth_config -> state:string -> result Lwt.t
 
 let pre_login_handlers = ref []
 
-let get_pre_login_handler uuid cont a =
-  let%lwt () = Eliom_reference.set auth_env (Some (uuid, a, cont)) in
+let get_pre_login_handler uuid kind a =
+  let%lwt state = generate_token () in
+  let%lwt () = Eliom_reference.set auth_env (Some (uuid, a, kind, state)) in
   match List.assoc_opt a.auth_system !pre_login_handlers with
-  | Some handler -> handler a
+  | Some handler -> handler a ~state
   | None -> fail_http 404
 
-let register_pre_login_handler auth_system handler =
-  pre_login_handlers := (auth_system, handler) :: !pre_login_handlers
+let register_pre_login_handler ~auth_system handler =
+  pre_login_handlers := (auth_system, handler) :: !pre_login_handlers;
+  run_post_login_handler ~auth_system
 
 let rec find_auth_instance x = function
   | [] -> None
   | { auth_instance = i; _ } as y :: _ when i = x -> Some y
   | _ :: xs -> find_auth_instance x xs
 
-let get_cont login_or_logout x =
-  let open Eliom_registration in
-  let redir = match x with
-    | `Election uuid -> Redirection (preapply election_cast uuid)
-    | `Site ContSiteHome -> Redirection home
-    | `Site ContSiteAdmin -> Redirection admin
-    | `Site (ContSiteElection uuid) ->
-       match login_or_logout with
-       | `Login -> Redirection (preapply election_admin uuid)
-       | `Logout -> Redirection (preapply election_home (uuid, ()))
-  in
-  fun () -> Redirection.send redir
-
 let login_handler service kind =
   let uuid = match kind with
     | `Site _ -> None
@@ -86,19 +101,23 @@ let login_handler service kind =
   in
   let myself service =
     match kind with
-    | `Site cont -> preapply site_login (service, cont)
-    | `Election uuid -> preapply election_login ((uuid, ()), service)
+    | `Site cont -> preapply ~service:site_login (service, cont)
+    | `Election uuid -> preapply ~service:election_login ((uuid, ()), service)
   in
   let%lwt user = match uuid with
     | None -> Eliom_reference.get Web_state.site_user
     | Some uuid -> Web_state.get_election_user uuid
   in
-  match user with
-  | Some _ -> get_cont `Login kind ()
-  | None ->
+  match user, uuid with
+  | Some _, None -> get_cont `Login kind ()
+  | Some _, Some _ | None, _ ->
      let%lwt c = match uuid with
        | None -> return !Web_config.site_auth_config
-       | Some u -> Web_persist.get_auth_config u
+       | Some uuid ->
+          let%lwt metadata = Web_persist.get_election_metadata uuid in
+          match metadata.e_auth_config with
+          | None -> return []
+          | Some x -> return x
      in
      match service with
      | Some s ->
@@ -107,8 +126,7 @@ let login_handler service kind =
           | Some x -> return x
           | None -> fail_http 404
         in
-        let cont = get_cont `Login kind in
-        get_pre_login_handler uuid cont a
+        get_pre_login_handler uuid kind a
      | None ->
         match c with
         | [s] -> Eliom_registration.(Redirection.send (Redirection (myself (Some s.auth_instance))))
@@ -116,11 +134,11 @@ let login_handler service kind =
            let builder =
              match kind with
              | `Site cont -> fun s ->
-               preapply Web_services.site_login (Some s, cont)
+               preapply ~service:Web_services.site_login (Some s, cont)
              | `Election uuid -> fun s ->
-               preapply Web_services.election_login ((uuid, ()), Some s)
+               preapply ~service:Web_services.election_login ((uuid, ()), Some s)
            in
-           Web_templates.login_choose (List.map (fun x -> x.auth_instance) c) builder () >>=
+           Pages_common.login_choose (List.map (fun x -> x.auth_instance) c) builder () >>=
            Eliom_registration.Html.send
 
 let logout_handler cont =
