@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -26,7 +26,7 @@ open Web_common
 
 let scope = Eliom_common.default_session_scope
 
-let oidc_state = Eliom_reference.eref ~scope None
+let oidc_config = Eliom_reference.eref ~scope None
 
 let login_oidc = Eliom_service.create
   ~path:(Eliom_service.Path ["auth"; "oidc"])
@@ -36,7 +36,7 @@ let login_oidc = Eliom_service.create
 let oidc_self =
   lazy (Eliom_uri.make_string_uri
           ~absolute:true
-          ~service:(preapply login_oidc [])
+          ~service:(preapply ~service:login_oidc [])
           () |> rewrite_prefix)
 
 let oidc_get_userinfo ocfg info =
@@ -71,28 +71,6 @@ let oidc_get_name ocfg client_id client_secret code =
     oidc_get_userinfo ocfg info
   | None -> return_none
 
-let oidc_handler params () =
-  Web_auth.run_post_login_handler "oidc" (fun _ _ authenticate ->
-      let code = List.assoc_opt "code" params in
-      let state = List.assoc_opt "state" params in
-      match code, state with
-      | Some code, Some state ->
-         let%lwt ocfg, client_id, client_secret, st =
-           match%lwt Eliom_reference.get oidc_state with
-           | None -> failwith "oidc handler was invoked without a state"
-           | Some x -> return x
-         in
-         let%lwt () = Eliom_reference.unset oidc_state in
-         if state <> st then fail_http 401 else
-           (match%lwt oidc_get_name ocfg client_id client_secret code with
-            | Some name -> authenticate name
-            | None -> fail_http 401
-           )
-      | _, _ -> return_unit
-    )
-
-let () = Eliom_registration.Any.register ~service:login_oidc oidc_handler
-
 let get_oidc_configuration server =
   let url = server ^ "/.well-known/openid-configuration" in
   let%lwt reply = Ocsigen_http_client.get_url url in
@@ -108,13 +86,12 @@ let split_prefix_path url =
   let i = String.rindex url '/' in
   String.sub url 0 i, [String.sub url (i+1) (n-i-1)]
 
-let oidc_login_handler a =
+let oidc_login_handler a ~state =
   let get x = List.assoc_opt x a.auth_config in
-  match get "server", get "client_id", get "client_secret" with
-  | Some server, Some client_id, Some client_secret ->
+  match get "server", get "client_id" with
+  | Some server, Some client_id ->
      let%lwt ocfg = get_oidc_configuration server in
-     let%lwt state = generate_token () in
-     let%lwt () = Eliom_reference.set oidc_state (Some (ocfg, client_id, client_secret, state)) in
+     let%lwt () = Eliom_reference.set oidc_config (Some ocfg) in
      let prefix, path = split_prefix_path ocfg.authorization_endpoint in
      let auth_endpoint = Eliom_service.extern ~prefix ~path
        ~meth:(Eliom_service.Get Eliom_parameter.(string "redirect_uri" **
@@ -122,10 +99,37 @@ let oidc_login_handler a =
            string "scope" ** string "state" ** string "prompt"))
        ()
      in
-     let service = preapply auth_endpoint
+     let service = preapply ~service:auth_endpoint
        (Lazy.force oidc_self, ("code", (client_id, ("openid email", (state, "consent")))))
      in
      Eliom_registration.(Redirection.send (Redirection service))
   | _ -> failwith "oidc_login_handler invoked with bad config"
 
-let () = Web_auth.register_pre_login_handler "oidc" oidc_login_handler
+let run_post_login_handler =
+  Web_auth.register_pre_login_handler ~auth_system:"oidc" oidc_login_handler
+
+let oidc_handler params () =
+  let code = List.assoc_opt "code" params in
+  let state = List.assoc_opt "state" params in
+  match code, state with
+  | Some code, Some state ->
+     run_post_login_handler ~state
+       (fun _ a authenticate ->
+         let get x = List.assoc_opt x a.auth_config in
+         match get "client_id", get "client_secret" with
+         | Some client_id, Some client_secret ->
+            let%lwt ocfg =
+              match%lwt Eliom_reference.get oidc_config with
+              | None -> failwith "oidc handler was invoked without discovered configuration"
+              | Some x -> return x
+            in
+            let%lwt () = Eliom_reference.unset oidc_config in
+            (match%lwt oidc_get_name ocfg client_id client_secret code with
+             | Some name -> authenticate name >>= fun x -> return (Ok x)
+             | None -> return (Error ())
+            )
+         | _, _ -> fail_http 503
+       )
+  | _, _ -> fail_http 401
+
+let () = Eliom_registration.Any.register ~service:login_oidc oidc_handler
