@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -20,6 +20,7 @@
 (**************************************************************************)
 
 let site_auth_config = ref []
+let locales_dir = ref "."
 let spool_dir = ref "."
 let server_mail = ref "noreply@example.org"
 let return_path = ref None
@@ -31,3 +32,4 @@ let maxmailsatonce = ref 1000
 let uuid_length = ref None
 let default_group = ref ""
 let nh_group = ref ""
+let domain = ref ""
