@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -22,6 +22,7 @@
 open Web_serializable_t
 
 val site_auth_config : auth_config list ref
+val locales_dir : string ref
 val spool_dir : string ref
 val server_mail : string ref
 val return_path : string option ref
@@ -33,3 +34,4 @@ val maxmailsatonce : int ref
 val uuid_length : int option ref
 val default_group : string ref
 val nh_group : string ref
+val domain : string ref
