@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2012-2018 Inria                                           *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -20,26 +20,25 @@
 (**************************************************************************)
 
 open Lwt
+open Belenios
 open Serializable_builtin_t
 open Web_serializable_j
 open Web_common
 
-(** Global initialization *)
-
-let () = CalendarLib.Time_Zone.(change Local)
-
 (** Parse configuration from <eliom> *)
 
+let locales_dir = ref None
 let spool_dir = ref None
 let source_file = ref None
 let auth_instances = ref []
 let gdpr_uri = ref None
 let default_group_file = ref None
 let nh_group_file = ref None
+let domain = ref None
 
 let () =
   Eliom_config.get_config () |>
-  let open Simplexmlparser in
+  let open Xml in
   List.iter @@ function
   | PCData x ->
     Ocsigen_extensions.Configuration.ignore_blank_pcdata ~in_tag:"belenios" x
@@ -75,6 +74,8 @@ let () =
      in
      set "mail" (fun x -> Web_config.server_mail := x);
      set "return-path" (fun x -> Web_config.return_path := Some x);
+  | Element ("locales", ["dir", dir], []) ->
+    locales_dir := Some dir
   | Element ("spool", ["dir", dir], []) ->
     spool_dir := Some dir
   | Element ("warning", ["file", file], []) ->
@@ -85,6 +86,8 @@ let () =
              [Element (auth_system, auth_config, [])]) ->
     let i = {auth_system; auth_instance; auth_config} in
     auth_instances := i :: !auth_instances
+  | Element ("domain", ["name", name], []) ->
+     domain := Some name
   | Element (tag, _, _) ->
     Printf.ksprintf failwith
       "invalid configuration for tag %s in belenios"
@@ -97,45 +100,64 @@ let () =
 
 (** Parse configuration from other sources *)
 
-let%lwt source_file =
-  match !source_file with
-  | Some f ->
-    let%lwt b = file_exists f in
-    if b then (
-      return f
-    ) else (
-      Printf.ksprintf failwith "file %s does not exist" f
+let source_file =
+  Lwt_main.run
+    (match !source_file with
+     | Some f ->
+        let%lwt b = file_exists f in
+        if b then (
+          return f
+        ) else (
+          Printf.ksprintf failwith "file %s does not exist" f
+        )
+     | None -> failwith "missing <source> in configuration"
     )
-  | None -> failwith "missing <source> in configuration"
+
+let locales_dir =
+  match !locales_dir with
+  | Some d -> d
+  | None -> failwith "missing <locales> in configuration"
 
 let spool_dir =
   match !spool_dir with
   | Some d -> d
   | None -> failwith "missing <spool> in configuration"
 
-let%lwt default_group =
-  match !default_group_file with
-  | None -> failwith "missing <default-group> in configuration"
-  | Some x ->
-     let%lwt x = Lwt_io.lines_of_file x |> Lwt_stream.to_list in
-     match x with
-     | [x] -> return x
-     | _ -> failwith "invalid default group file"
+let default_group =
+  Lwt_main.run
+    (match !default_group_file with
+     | None -> failwith "missing <default-group> in configuration"
+     | Some x ->
+        let%lwt x = Lwt_io.lines_of_file x |> Lwt_stream.to_list in
+        match x with
+        | [x] -> return x
+        | _ -> failwith "invalid default group file"
+    )
 
-let%lwt nh_group =
-  match !nh_group_file with
-  | None -> failwith "missing <nh-group> in configuration"
-  | Some x ->
-     let%lwt x = Lwt_io.lines_of_file x |> Lwt_stream.to_list in
-     match x with
-     | [x] -> return x
-     | _ -> failwith "invalid NH group file"
+let nh_group =
+  Lwt_main.run
+    (match !nh_group_file with
+     | None -> failwith "missing <nh-group> in configuration"
+     | Some x ->
+        let%lwt x = Lwt_io.lines_of_file x |> Lwt_stream.to_list in
+        match x with
+        | [x] -> return x
+        | _ -> failwith "invalid NH group file"
+    )
+
+let domain =
+  match !domain with
+  | Some d -> d
+  | None -> failwith "missing <domain> in configuration"
 
 (** Build up the site *)
 
 let () = Web_config.source_file := source_file
+let () = Web_config.locales_dir := locales_dir
 let () = Web_config.spool_dir := spool_dir
 let () = Web_config.default_group := default_group
 let () = Web_config.nh_group := nh_group
 let () = Web_config.site_auth_config := List.rev !auth_instances
-let () = Lwt.async Web_site.data_policy_loop
+let () = Web_config.domain := domain
+let () = Lwt_main.run (Web_persist.convert_trustees ())
+let () = Lwt.async Site_admin.data_policy_loop
