@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                BELENIOS                                *)
 (*                                                                        *)
-(*  Copyright © 2018 Inria                                                *)
+(*  Copyright © 2012-2020 Inria                                           *)
 (*                                                                        *)
 (*  This program is free software: you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU Affero General Public License as        *)
@@ -19,6 +19,8 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
+open Belenios_platform
+open Belenios
 open Platform
 open Common
 open Web_serializable_builtin_t
@@ -112,7 +114,7 @@ let send_confirmation_link ~service address =
     Eliom_uri.make_string_uri ~absolute:true ~service:Web_services.signup_login
       token |> rewrite_prefix
   in
-  let message =
+  let body =
     Printf.sprintf "\
 Dear %s,
 
@@ -131,7 +133,8 @@ Best regards,
 -- \n\
 Belenios Server" address uri
   in
-  let%lwt () = send_email address "Belenios account creation" message in
+  let subject = "Belenios account creation" in
+  let%lwt () = send_email MailAccountCreation ~recipient:address ~subject ~body in
   Lwt.return_unit
 
 let send_changepw_link ~service ~address ~username =
@@ -145,7 +148,7 @@ let send_changepw_link ~service ~address ~username =
     Eliom_uri.make_string_uri ~absolute:true ~service:Web_services.signup_login
       token |> rewrite_prefix
   in
-  let message =
+  let body =
     Printf.sprintf "\
 Dear %s,
 
@@ -162,18 +165,22 @@ address are no longer valid.
 Best regards,
 
 -- \n\
-Belenios Server" username uri
+Belenios Server" address uri
   in
-  let%lwt () = send_email address "Belenios password change" message in
+  let subject = "Belenios password change" in
+  let%lwt () = send_email MailPasswordChange ~recipient:address ~subject ~body in
   Lwt.return_unit
 
 let confirm_link token =
   links := filter_links_by_time !links;
   match SMap.find_opt token !links with
   | None -> Lwt.return_none
-  | Some x ->
-     links := SMap.remove token !links;
-     Lwt.return_some (x.service, x.address, x.kind)
+  | Some x -> Lwt.return_some (token, x.service, x.address, x.kind)
+
+let remove_link token =
+  links := filter_links_by_time !links;
+  links := SMap.remove token !links;
+  Lwt.return_unit
 
 let cracklib =
   let x = "cracklib-check" in (x, [| x |])
